/* 
 *   Biblioteca AdjList.h
 *   Autor: Moacir P Ponti Jr. ICMC/USP 2011
 * 
 *   Implementa uma lista de adjacencia para representacao de grafos
 *
 *   Manipula um ponteiro para estrutura digraph (tipos Digraph ou Graph) que contem:
 *	- V: o número de vértices
 *	- A: o número de arestas
 *	- list: um arranjo de vértices, cada qual com um ponteiro para uma lista de adjacencias
 *
 *   Possui funcoes para:
 *	- inicializacao de grafo: initGraph() initDigraph()
 *	- geracao aleatoria de grafo: randomGraph() randomDigraph()
 *	- insercao e remocao de arestas: insertEdge() insertArc() removeEdge() removeArc()
 *	- busca em largura e em profundidade: bfsAllDigraph() bfsDigraph() dfsAllGraph() dfsGraph()
 */

#include <stdlib.h>
#include <stdio.h>
#include <float.h>

// Vértices são representados por objetos do tipo Vertex.  
// Os vértices de um digrafo são 0,1,..,V-1. */
#define Vertex int
static int count;

/* Um 'node' representa um elemento na lista de adjacencia
   e o tipo 'Link' e um ponteiro para a estrutura 'node  */
typedef struct node *Link;

/* VertexList
   visited: -1 se o vertice nao foi visitado ou um numero >= 0 se foi visitado
   adj: ponteiro para uma lista de adjacencias (lista encadeada de 'nodes')      */
typedef struct {
  Link adj;
  int visited;
} VertexList;

/* node - um elemento da lista de adjacencia
   w - indice do vertice adjacente
   weight - peso da aresta relativo a incidencia em w
   next - ponteiro para o proximo elemento da lista de adjacencia  */
struct node {
  Vertex w;      
  double weight; 
  Link next;     
};

/* digraph - estrutura que contem o digrafo/grafo
   V - numero de vertices
   A - numero de arcos/arestas
   list - ponteiro para um arranjo de vertices (VertexList) */
struct digraph {
   int V;
   int A;
   VertexList *list;
};

// Digraph sera um ponteiro para a criacao de um digrafo
typedef struct digraph *Digraph;

/*  NewArc recebe um vertice w e o endereco nnext
    retorna um novo elemento 'x' para a lista de adjacencia
    de forma a inserir o elemento no inicio (cabeca) da lista  */
Link NewArc(Vertex nw, Link nnext, double weight) {
    Link x = malloc (sizeof(*x)); // malloc (sizeof(struct node)) tambem funciona
    x->w = nw;
    x->weight = weight;
    x->next = nnext;
    return x;
}

#define graph digraph
#define Graph Digraph

/*  initDigraph - inicializa o grafo com um numero de vertices determinado
    retorna um ponteiro para uma estrutura digraph   */
Digraph initDigraph(int V) {
  int v;
  // aloca dinamicamente uma estrutura digrafo e inicializa variaveis
  Digraph G = (Digraph)malloc(sizeof(*G));
  G->V = V;
  G->A = 0;
  // cria lista de vertices com V elementos
  G->list = (VertexList *) malloc(V* sizeof(VertexList));
  
  // cada elemento da lista sera inicializado como nao visitado 
  // e cada lista de adjacencia como nulo
  for (v=0; v < V; v++) {
      G->list[v].visited = -1;
      G->list[v].adj = NULL;
  }
  return G;
}

/*  insertArc - insere um arco num grafo
      requer: um digrafo G, um par de vertices v,w
              e um peso para a aresta  */
void insertArc(Digraph G, Vertex v, Vertex w, double weight) {
  Link p;
  // evita lacos
  if (v == w) return;
  if ((v > G->V-1) || (w > G->V-1)) return;
  
  // percorre a lista de adjacencia para verificar duplicidade
  for (p=G->list[v].adj; p!=NULL; p = p->next) {
      if (p->w == w) return;
  }
      
  // insere arco no inicio da lista de adjacencia
  G->list[v].adj = NewArc(w, G->list[v].adj, weight);
  G->A++;
}

/*  removeArc - remove um arco de um grafo
      requer: um grafo G, um par de vertices v,w     */
void removeArc(Digraph G, Vertex v, Vertex w) {
  Link p, prevp;
  // evita lacos
  if (v == w) return;
  if ((v > G->V-1) || (w > G->V-1)) return;

  // dois ponteiros, um para armazenar o anterior (prevp) e outro o atual (p)
  prevp = G->list[v].adj;
  p = G->list[v].adj;
  
  while (p!=NULL) {
      if (p->w == w) {
	 // se o elemento a remover for o primeiro da lista
	 if (p == G->list[v].adj) {
	    G->list[v].adj = p->next;
            G->A--;
	    free(p);
	    return;
	 } 
	 // se o elemento a remover estiver depois do primeiro da lista
	 else {
	   prevp->next = p->next;
	   free(p);
	   G->A--;
	   return;
	 }
      } 
      // move o ponteiro (p) para o proximo elemento, guardando o anterior (prevp)
      prevp = p;
      p = p->next;
  }
}

/*  printDigraph - imprime os vertices e adjacentes de um digrafo  */
void printDigraph(Digraph G) {
  Link p;
  Vertex i;
  
  printf("\nGraph vertices and adjacences:");
  for (i=0;i<G->V;i++) {
    printf("\nVertex %d (%d):\n", i, G->list[i].visited);
    // percorre a lista de adjacencia de cada vertice 'i'
    for (p=G->list[i].adj; p!=NULL; p = p->next) {
	printf("   -> %d (peso %.1lf)\n", p->w, p->weight);
	if (p->next == NULL)
	   printf("   (fim)\n");
    }
  }
  printf("\n");
}

/*  printVisitedDigraph - imprime os vertices que foram visitados por um algoritmo de percurso */
void printVisitedDigraph(Digraph G) {
  Vertex i;
  
  printf("\nGraph visited vertices:\n");
  for (i=0; i < G->V; i++) {
    if (G->list[i].visited >= 0) {
       printf("-> vertex %d (number %d)\n", i, G->list[i].visited);
    }
  }
  printf("\n");
}

/*  definicoes para Graph ao inves de Digraph */
#define printVisitedGraph printVisitedDigraph
#define printGraph printDigraph
#define initGraph initDigraph

/* insertEdge - insere uma aresta em um grafo
  usa a ideia de que a insercao de uma aresta corresponde a insercao de 
  dois arcos anti-paralelos
  requer: um grafo G, um par de vertices v,w
          e um peso para a aresta  */
void insertEdge(Graph G, Vertex v, Vertex w, double weight) {
    insertArc(G, v, w, weight); 
    insertArc(G, w, v, weight); 
}

/* removeEdge - remove uma aresta de um grafo
  usa a ideia de que a remocao de uma aresta corresponde a remocao de 
  dois arcos anti-paralelos
  requer: um grafo G, um par de vertices v,w                            */
void removeEdge(Graph G, Vertex v, Vertex w) {
    removeArc(G, v, w); 
    removeArc(G, w, v); 
}

/* randomGraph - inicializa e gera um grafo aleatorio com V vertices e E arestas
   requer: numero de vertices V e o numero de arestas E 
*/
Graph randomGraph(int V, int E) {
   Graph G = initGraph(V);
   while (G->A <= E) {
       Vertex v = rand()%G->V;
       Vertex w = rand()%G->V;
       double weight = (double)(rand()%100);
       if (v == w) continue;
       insertEdge(G,v,w,weight);
   }
   return G;
}

/* randomDigraph - inicializa e gera um digrafo aleatorio com V vertices e E arcos
   requer: numero de vertices V e o numero de arcos E 
*/
Digraph randomDigraph(int V, int E) {
   Digraph G = initDigraph(V);
   while (G->A <= E) {
       Vertex v = rand()%G->V;
       Vertex w = rand()%G->V;
       double weight = (double)(rand()%100);
       if (v == w) continue;
       insertArc(G,v,w,weight);
   }
   return G;
}

/* DFS - realiza busca em profundidade
   requer: um digrafo/grafo G e um vertice inicial v
   sera usada apenas dentro da biblioteca, o usuario 
   deve chamar uma das outras funcoes:
     - dfsAllDigraph
     - dfsDigraph
*/
void DFS(Digraph G, Vertex v) { 
   Link p; 
   // visitar v
   
   // marca v como visitado
   G->list[v].visited = count++;
   printf(">> DFS %d:\n", v);
   
   // percorrer os adjacentes a v
   for (p = G->list[v].adj; p != NULL; p = p->next) {
      // se nao tiver sido visitado, executa DFS
      if (G->list[p->w].visited == -1) {
         printf("    encontrou %d-%d e ira visitar %d\n", v, p->w, p->w);
         DFS(G, p->w);
      } else {
         printf("    encontrou (%d) que ja foi visitado\n", p->w);
      }
   }
   printf("<< DFS %d:\n", v);
}

/* dfsAllDigraph - realiza busca em profundidade em todo o digrafo
   requer: um digrafo/grafo G 
   - faz a busca percorrendo todo o grafo mesmo que esse
     NAO seja conexo
   - usa uma variavel global 'cont' para guardar a ordem que 
     percorreu os vertices
   - pode ser chamado de fora da biblioteca
*/
void dfsAllDigraph(Digraph G) { 
   Vertex v;
   count = 0; // inicializa a ordem do percurso
   for (v=0; v < G->V; v++) {
      G->list[v].visited = -1;
   }
   
   // roda para todos os vértices iniciais
   for (v = 0; v < G->V; v++)
      if (G->list[v].visited == -1) 
         DFS(G, v);
}

/* dfsDigraph - realiza busca em profundidade com inicio em v
   requer: um digrafo/grafo G e um vertice inicial v
   - faz a busca percorrendo o grafo a partir de v
   - usa uma variavel global 'cont' para guardar a ordem que 
     percorreu os vertices
   - pode ser chamado de fora da biblioteca
*/
void dfsDigraph(Digraph G, Vertex v) { 
   Vertex a;
   count = 0;
   for (a=0; a < G->V; a++) {
      G->list[a].visited = -1;
   }
   
   DFS(G, v);
}

/* definicao para uso de DFS em grafos (Graph )*/
#define dfsGraph dfsDigraph
#define dfsAllGraph dfsAllDigraph

void BFS(Digraph G, Vertex v) { 
   Link p; 
   // visitar v
   G->list[v].visited = count++; // armazena a ordem em que os vertices foram visitados

   int *q = (int *) malloc(sizeof(int)*G->V); // cria uma fila circular para armazenar os proximos vertices a serem visitados
   int qini = 0, qfim = 0; // indices para o inicio e fim da fila
   
   q[qfim++] = v;  // insere v na fila
   
   // enquanto fila for NAO vazia
   while (qfim!=qini) {
      v = q[qini]; // retira elemento do inico da fila
      printf(">> BFS %d:\n", v);
      qini = (qini + 1)%(G->V); // atualiza indice de inicio da fila
      
      for (p = G->list[v].adj; p != NULL; p = p->next) {
	
	  // se um vertice w adjacente ainda nao foi visitado insere na fila
	  if (G->list[p->w].visited == -1) {
	     printf("    encontrou %d-%d, marca %d como visitado e insere na fila\n", v, p->w, p->w);
	     G->list[p->w].visited = count++;
	     q[qfim] = p->w;
	     qfim = (qfim + 1)%(G->V); // atualiza indice de inicio da fila
	  } else {
	    printf("    encontrou (%d) que ja foi visitado\n", p->w);
	  }
      }
      printf("<< BFS %d (inicio %d fim %d):\n", v, qini, qfim);
   }
   
   free(q); // libera memoria alocada pela fila
}

/* dfsAllDigraph - realiza busca em profundidade em todo o digrafo
   requer: um digrafo/grafo G 
   - faz a busca percorrendo todo o grafo mesmo que esse
     NAO seja conexo
   - usa uma variavel global 'cont' para guardar a ordem que 
     percorreu os vertices
   - pode ser chamado de fora da biblioteca
*/
void bfsAllDigraph(Digraph G) { 
   Vertex v;
   count = 0; // inicializa a ordem do percurso
   for (v=0; v < G->V; v++) {
      G->list[v].visited = -1;
   }
   
   // roda para todos os vértices iniciais
   for (v = 0; v < G->V; v++)
      if (G->list[v].visited == -1) 
         BFS(G, v);
}

/* dfsDigraph - realiza busca em profundidade com inicio em v
   requer: um digrafo/grafo G e um vertice inicial v
   - faz a busca percorrendo o grafo a partir de v
   - usa uma variavel global 'cont' para guardar a ordem que 
     percorreu os vertices
   - pode ser chamado de fora da biblioteca
*/
void bfsDigraph(Digraph G, Vertex v) { 
   Vertex a;
   count = 0;
   for (a=0; a < G->V; a++) {
      G->list[a].visited = -1;
   }
   
   BFS(G, v);
}

/* Define as funcoes bfsGraph e bfsAllGraph para uso  */
#define bfsGraph bfsDigraph
#define bfsAllGraph bfsAllDigraph

/* topologicalSort() - realiza busca em profundidade com inicio em v
   requer: um digrafo/grafo G e um arranjo de inteiros
   - calcula o grau de entrada de cada vertice e armazena em inDeg[]
   - usa uma fila para guardar os vertices fonte (cujo inDeg==0)
   - retira os vertices fonte, insere no arranjo ordenado e atualiza o grafo
*/
int topologicalSort(Digraph G, int *ts) { 
   Link p;
   Vertex v;
   int i;
   // visitar v

   // arranjo com os graus de entrada de cada vertice
   int *inDeg = (int *) calloc(G->V, sizeof(int)); 

   // calcula grau de entrada de cada vertice
   for (v = 0; v < G->V; v++) 
      for (p = G->list[v].adj; p != NULL; p = p->next) 
          inDeg[p->w]++;
   
   // fila para guardar os vertices fonte
   int *q = (int *) calloc(G->V, sizeof(int)); 
   int qini = 0, qfim = 0; // indices para o inicio e fim da fila
   
   // procura vertices fonte e os armazena na fila
   for (v = 0; v < G->V; v++) 
       if (inDeg[v] == 0) 
	  q[qfim++] = v;  // insere v na fila

   // executa ate que a fila seja vazia
   for (i = 0; qfim!=qini; i++) {
	ts[i] = v = q[qini++]; // retira vertice da fila e insere na lista ordenada
	for (p = G->list[v].adj; p != NULL; p = p->next) {
	    inDeg[p->w]--; // decrementa grau de entrada dos vertices adjacentes a v
	    // se o vertice se tornou fonte, insere na fila
	    if (inDeg[p->w] == 0) {
	       q[qfim++] = p-> w;  // insere v na fila
	    }    
	}
   }
   free(q); // libera memoria alocada pela fila
   free(inDeg);
   return ( i != G->V );
}



/* dijkstra() - resolve o problema de encontrar o caminho mais barato
   a partir de um dado vertice 
   > requer: um digrafo/grafo G 
   - um digrafo/grafo G
   - um vertice inicial s
   - um vetor de distancias distance
   - um vetor de predecessores parent
*/
void dijkstra(Digraph G, Vertex s, double *distance, Vertex *parent) { 
   
   int i, j;
   Vertex v, w;
   double c_weight;
   double s_dist;

   // controle da construcao da arvore
   int intree[G->V];
   
   // inicializacao
   for (i=0; i < G->V; i++) {
       distance[i] = DBL_MAX;
       parent[i] = -1;
       intree[i] = 0;
   }
   
   distance[s] = 0;
   v = s;
   
   // enquanto houver um vertice nao inserido na arvore
   // inicialmente esse vertice sera o inicial
   while (intree[v] == 0) {
     
      // registra como ja visitado (inserido na arvore)
      intree[v] = 1;
      Link p;
      
      // percorre a lista de adjacencia do vertice visitado
      // e atualiza distancias se for o caso
      for (p = G->list[v].adj; p != NULL; p = p->next) {
          w = p->w;
	  c_weight = p->weight;
	  if (distance[w] > (distance[v]+c_weight)) {
	       distance[w] = distance[v]+c_weight;
	       parent[w] = v;
	  }
      }      
      
      // encontra o proximo vertice a processar
      v = 0;
      s_dist = DBL_MAX;
      for (i = 0; i < G->V; i++) {
          // se o vertice ainda nao estiver na arvore e sua
	  // distancia for maior do que a maior distancia ja econtrada
	  // entao esse vertice sera o proximo a ser inserido na arvore
	  if ((intree[i]==0) && (s_dist > distance[i])) {
	     s_dist = distance[i];
	     v = i;
	  }
      }
   }
}


/* prim() - resolve o problema de encontrar uma arvore geradora minima
   requer: um digrafo/grafo G 
   - um digrafo/grafo G
   - um vertice inicial s
   - um vetor de distancias distance
   - um vetor de predecessores parent
*/
void prim(Digraph G, Vertex s, double *distance, Vertex *parent) { 
   
   int i, j;
   Vertex v, w;
   double c_weight;
   double s_dist;

   // controle da construcao da arvore
   int intree[G->V];
   
   // inicializacao
   for (i=0; i < G->V; i++) {
       distance[i] = DBL_MAX;
       parent[i] = -1;
       intree[i] = 0;
   }
   
   distance[s] = 0;
   v = s;
   
   // enquanto houver um vertice nao inserido na arvore
   // inicialmente esse vertice sera o inicial
   while (intree[v] == 0) {
     
      // registra como ja visitado (inserido na arvore)
      intree[v] = 1;
      Link p;
      
      // percorre a lista de adjacencia do vertice visitado
      // e atualiza distancias se for o caso
      for (p = G->list[v].adj; p != NULL; p = p->next) {
          w = p->w;
	  c_weight = p->weight;
	  // se o vertice ainda nao inserido na arvore e tiver aresta
	  // com menor peso, atualiza o peso
	  if ((distance[w] > c_weight) && (intree[w] == 0 )) {
	       distance[w] = c_weight;
	       parent[w] = v;
	  }
      }      
      
      // encontra o proximo vertice a processar
      v = 0;
      s_dist = DBL_MAX;
      for (i = 0; i < G->V; i++) {
          // se o vertice ainda nao estiver na arvore e sua
	  // distancia for maior do que a maior distancia ja econtrada
	  // entao esse vertice sera o proximo a ser inserido na arvore
	  if ((intree[i]==0) && (s_dist > distance[i])) {
	     s_dist = distance[i];
	     v = i;
	  }
      }
   }
}

